from typing import Union, List

from sqlalchemy.exc import NoResultFound

from kclii.database.database import session
from kclii.error.database import NotRecordFound
from .models import ProfileModel, EnvironmentVariablesModel
from .core import ProfileCoreIn, EnvironmentVariablesIn


class Repository:
    def get_all(self) -> List[ProfileModel]:
        return session.query(ProfileModel).all()

    def get_profile_by_name(self, name: str) -> Union[ProfileModel, NoResultFound]:
        try:
            saved_profile = (
                session.query(ProfileModel).filter(ProfileModel.name == name).one()
            )
        except NoResultFound as error:
            raise NotRecordFound(filter_by={ProfileModel.name, name}) from error

        return saved_profile

    def create_profile(self, profile: ProfileCoreIn) -> ProfileModel:
        new_profile = ProfileModel(name=profile.name)

        session.add(new_profile)
        session.commit()

        return new_profile

    def add_environment_variables(
        self, profile_id: int, environment_variables: List[EnvironmentVariablesIn]
    ):
        environment_variables = [
            EnvironmentVariablesModel(
                key=environment_variable.key,
                value=environment_variable.value,
                profile_id=profile_id,
            )
            for environment_variable in environment_variables
        ]

        session.add_all(environment_variables)
        session.commit()

    def delete_environment_variables(
        self,
        delete_ids: List[int],
    ):
        session.query(EnvironmentVariablesModel).filter(
            EnvironmentVariablesModel.id.in_(delete_ids)
        ).delete()
        session.commit()

    def get_environment_variables_by_profile(
        self, profile_id: int
    ) -> Union[EnvironmentVariablesModel, NoResultFound]:
        try:
            saved_environment_variables = (
                session.query(EnvironmentVariablesModel)
                .filter(EnvironmentVariablesModel.profile_id == profile_id)
                .all()
            )
        except NoResultFound as error:
            raise NotRecordFound(
                filter_by={EnvironmentVariablesModel.profile_id, profile_id}
            ) from error

        return saved_environment_variables



from .kon import SCRIPT as kon
from .koff import SCRIPT as koff


def install_scripts() -> None:
    with open("/usr/local/bin/kon.sh", "w+") as file:
        file.write(kon)

    with open("/usr/local/bin/koff.sh", "w+") as file:
        file.write(koff)



import os
from typing import List, Tuple, Optional

import typer
from rich.console import Console
from rich.table import Table

from kclii.consts.envs import K_DEFAULT_PROFILE
from kclii.helper.message import new_message
from kclii.helper.data import request_value
from kclii.envs.envs import set_env
from .core import EnvironmentVariablesIn, ProfileCoreIn
from .repository import Repository


TITLE = "👤 Profile"

app = typer.Typer()
repository = Repository()
console = Console()


def request_profile_data(
    skip_name: Optional[bool] = False,
    skip_environment_variables: Optional[bool] = False,
    force: Optional[bool] = False,
) -> Tuple[str, List[EnvironmentVariablesIn]]:
    current_profile = os.getenv(K_DEFAULT_PROFILE)

    if current_profile and not force:
        name = current_profile
    elif not skip_name:
        name = request_value(
            message="Profile name",
            description="Type the profile name should be unique",
            example="company",
        )
    else:
        name = ""

    if not skip_environment_variables:
        environment_variables = request_value(
            description="Add all the environments variables you need to this profile.",
            message="Environment Variables",
            example="MY_TOKEN=thisismyawesometoken!",
            rules="Once you finish please type stop",
            multiple=True,
            key_value=True,
        )

        environment_variables = [
            EnvironmentVariablesIn(key=list(env.keys())[0], value=list(env.values())[0])
            for env in environment_variables
        ]
    else:
        environment_variables = []

    return name, environment_variables


def print_list_environment_variables_by_profile(name: str) -> None:
    saved_profile = repository.get_profile_by_name(name=name)
    saved_environment_variables = repository.get_environment_variables_by_profile(
        profile_id=saved_profile.id
    )

    table = Table("ID", "Name", "Value")
    for saved_environment_variable in saved_environment_variables:

        table.add_row(
            str(saved_environment_variable.id),
            saved_environment_variable.key,
            saved_environment_variable.value,
        )

    console.print(table)


def print_list_profiles() -> None:
    saved_profiles = repository.get_all()

    table = Table("ID", "Name", "CreatedAt")
    for saved_profile in saved_profiles:

        table.add_row(
            str(saved_profile.id),
            saved_profile.name,
            str(saved_profile.created_at),
        )

    console.print(table)


@app.command()
def new() -> None:
    new_message(
        title=TITLE,
        status="info",
        title_type="bold",
        message="Create a new profile",
    )
    name, environment_variables = request_profile_data(force=True)
    profile = ProfileCoreIn(name=name, environment_variables=environment_variables)
    saved_profile = repository.create_profile(profile=profile)
    repository.add_environment_variables(
        profile_id=saved_profile.id, environment_variables=profile.environment_variables
    )


@app.command()
def update(add: Optional[bool] = False, delete: Optional[bool] = False) -> None:

    if add:
        new_message(
            title=TITLE,
            status="info",
            title_type="bold",
            message="Add environment variables",
        )
        name, environment_variables = request_profile_data()
        saved_profile = repository.get_profile_by_name(name=name)
        repository.add_environment_variables(
            profile_id=saved_profile.id, environment_variables=environment_variables
        )

    if delete:
        new_message(
            title=TITLE,
            status="info",
            title_type="bold",
            message="Delete environment variables",
        )
        name, _ = request_profile_data(skip_environment_variables=True)
        print_list_environment_variables_by_profile(name=name)
        delete_ids = input("Choose all the ids you want to delete (13,15,19): ")
        delete_ids = [int(id) for id in delete_ids.split(",")]
        repository.delete_environment_variables(delete_ids=delete_ids)


@app.command()
def ls() -> None:
    new_message(
        title=TITLE,
        status="info",
        title_type="bold",
        message="List all profiles",
    )

    print_list_profiles()


@app.command()
def ls_env() -> None:
    new_message(
        title=TITLE,
        status="info",
        title_type="bold",
        message="List all environment variables by pofile",
    )

    name, _ = request_profile_data(skip_environment_variables=True)
    print_list_environment_variables_by_profile(name=name)


@app.command("set")
def _set(name: str) -> None:
    saved_profile = repository.get_profile_by_name(name=name)
    new_message(
        title=TITLE,
        status="success",
        title_type="bold",
        message=f"{saved_profile.name} ACTIVE",
    )
    variables_to_set = [f"{K_DEFAULT_PROFILE}={saved_profile.name}\n"]

    for environment_variables in saved_profile.environment_variables:
        variables_to_set.append(
            f"{environment_variables.key}={environment_variables.value}\n"
        )

    set_env(values=variables_to_set)


@app.command()
def current() -> None:
    print(os.getenv(K_DEFAULT_PROFILE))



from typing import List

from pydantic import BaseModel


class EnvironmentVariablesIn(BaseModel):
    key: str
    value: str

    class Config:
        orm_mode = True


class EnvironmentVariables(BaseModel):
    key: str
    value: str

    class Config:
        orm_mode = True


class ProfileCoreIn(BaseModel):
    name: str
    environment_variables: List[EnvironmentVariables] = []

    class Config:
        orm_mode = True


class Profile(BaseModel):
    name: str
    environment_variables: List[EnvironmentVariables] = []

    class Config:
        orm_mode = True



from sqlalchemy import Column, String, Integer, Sequence, DateTime, ForeignKey
from sqlalchemy.sql import func
from sqlalchemy.orm import relationship

from kclii.database.database import Base


class ProfileModel(Base):
    __tablename__ = "profiles"

    id = Column(Integer, Sequence("profile_id_seq"), primary_key=True)
    name = Column(String, nullable=False)
    environment_variables = relationship(
        "EnvironmentVariablesModel",
        back_populates="profile",
        cascade="all, delete, delete-orphan",
    )

    created_at = Column(DateTime(timezone=True), server_default=func.now())
    updated_at = Column(DateTime(timezone=True), onupdate=func.now())


class EnvironmentVariablesModel(Base):
    __tablename__ = "environment_variables"

    id = Column(Integer, Sequence("profile_id_seq"), primary_key=True)
    key = Column(String, nullable=False)
    value = Column(String, nullable=False)

    profile_id = Column(Integer, ForeignKey("profiles.id"))
    profile = relationship("ProfileModel", back_populates="environment_variables")

    created_at = Column(DateTime(timezone=True), server_default=func.now())
    updated_at = Column(DateTime(timezone=True), onupdate=func.now())



import os

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

engine = create_engine(os.environ["K_DATABASE"])
Session = sessionmaker(bind=engine)
session = Session()

Base = declarative_base()



import os

import typer
from dotenv import load_dotenv

from kclii.database.database import Base, engine
from kclii.modules.profiles import profiles
from kclii.scripts.scripts import install_scripts

load_dotenv()
app = typer.Typer(pretty_exceptions_enable=False)
app.add_typer(profiles.app, name="profile")


@app.command()
def init():
    print("Init configurations")
    install_scripts()


@app.command()
def hello(name: str = ""):
    print(f"Welcome sr! {name}")


@app.command()
def version() -> None:
    print("0.1.6")


if __name__ == "__main__":
    if os.getuid() != 0:
        import kclii.modules.profiles.models

        Base.metadata.create_all(engine)
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kclii',
 'kclii.consts',
 'kclii.database',
 'kclii.envs',
 'kclii.error',
 'kclii.helper',
 'kclii.modules',
 'kclii.modules.profiles',
 'kclii.scripts']

package_data = \
{'': ['*']}

install_requires = \
['load-dotenv>=0.1.0,<0.2.0',
 'psycopg2-binary>=2.9.5,<3.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'sqlalchemy>=1.4.43,<2.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['k = kclii.main:app']}

setup_kwargs = {
    'name': 'kclii',
    'version': '0.1.6',
    'description': 'CLI multipurpose.',
    'long_description': '# K CLI',
    'author': 'Andres Garcia',
    'author_email': 'jose.andres.gm29@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from rich import print  # pylint: disable=W0622


def get_status_color(status: str) -> str:
    if status == "success":
        return "green"
    if status == "danger":
        return "red"
    if status == "info":
        return "#00d8ff"


def create_title(title: str, status: str, title_type: str) -> str:
    if not title:
        return ""

    beauty_title = f"[ {title} ]: "

    if status or title_type:
        need_space = " " if title_type else ""
        title_tag_start = f"[{title_type}{need_space}{get_status_color(status=status)}]"
        title_tag_end = title_tag_start[0] + "/" + title_tag_start[1::]

        beauty_title = f"{title_tag_start}{beauty_title}{title_tag_end}"

    return beauty_title


def new_message(
    message: str, title: str = "", status: str = "", title_type: str = ""
) -> None:
    title = create_title(title=title, status=status, title_type=title_type)
    print(f"{title}{message}")



from typing import List, Union

from rich import print  # pylint: disable=W0622
from rich.prompt import Prompt
from rich.panel import Panel

STOPS_WORDS = ["stop", "Stop", "STOP", "exit", "Exit"]


def request_value(
    message: str,
    description: str = "",
    example: str = "",
    rules: str = " - ",
    multiple: bool = False,
    key_value: bool = False,
) -> Union[List[str], str, List[dict]]:

    text = f"""
[yellow]Description[/yellow]: {description}
[yellow]Example[/yellow]: {example}
[yellow]Rules[/yellow]: {rules}
        """
    print(Panel(text, title=message))

    data = []
    while True:
        if key_value:
            key = Prompt.ask(f"> {message} [Key]")
            if _is_stop(value=key):
                break
            value = Prompt.ask(f"> {message} [Value]")

            data.append({key: value})

        else:
            value = Prompt.ask(f"{message} [Value]")
            if _is_stop(value=value):
                break

            data.append(value)

        if not multiple:
            return data[0]

    return data


def _is_stop(value: str) -> bool:
    if value in STOPS_WORDS:
        return True
    return False
